#include "dmmcgrath.h"
#include "c5utils.h"
#define NULL 0
#define MAXSCORE 2100000
#include "Configuration.h"
#include <iostream>
#include <map>
#include <vector>
#include <algorithm>

/*I wasn't sure if you wanted this in comments in the code or in a text file so i made both.

I enhnced the tree by adding a comprehensive move generator and eval function as well as a
transposition table.

My tree goes up to a height of 5 (root plus 4 lvls)


My evaluation function takes the opponents and players last moves and assigns a score
based on how many pieces each one has in a row minus how many of their pieces are blocked
plus how many pieces they have blocked and then subtracts the 2 scores.

I tried to make the move generator more intelligent by having it evaluate every move before it
made it and only picking the best move. However this turned out to cause the AI to take longer
than 7 seconds per move at a height of 4. I also tried to record the location of all the pieces
in the board in a vector instead of iterating through the board in a double for loop. The problem
with this was that having to pass the vector to the tree and then to each individual node
used up far too much memory so the double for loop ended up being exponentially more efficient.
*/

using namespace std;

dmmcgrath::Node::Node(){
  for(int i=0;i<NUMSQUARES;i++){
    for(int j=0;j<NUMSQUARES;j++){
      child_[i][j]=NULL;
    }
  }
}
struct playerMove{
	int score;
	int posx;
	int posy;
};
bool compare(playerMove first,playerMove second){
	return first.score>second.score;
}
//passes in the game board, the row and col of the last move the opponent made, the row and col of the last player move made, the node from which to create children,
//whomoves, the old hashkey to be used to generate the new one and the number of pieces currently on the board
void dmmcgrath::GameTree::generateMove(int board[][NUMSQUARES],int oprow,int opcol,int prow, int pcol,int lvl, Node* nn,int whomoves,int oldkey,int pieces){
	int newkey = 0;
	int hash;
	bool noPiece =true;
	int op = -whomoves;//determines the value of the opponents piece
	if(whomoves == 1){//used to determine which has value to use for the transposition table
		hash = 1; 
	}
	else{
		hash = 2;
	}
	 for(int i=0;i<NUMSQUARES;i++){ //loops through the entire board 
        for(int j=0;j<NUMSQUARES;j++){
			if(board[i][j] == 0){//only makes a move if the square is empty
			board[i][j] = op;//makes a move using the opponents piece
			int opUp =  numUp(board,i,j);//checks the number of pieces in all the different directions of the piece placed
			int opDown =  numDown(board,i,j);
			int opLeft =  numLeft(board,i,j);
			int opRight =  numRight(board,i,j);
			
			int opDownRight =  numDownRight(board,i,j);
			int opDownLeft  =  numDownLeft(board,i,j);
			int opUpRight   =  numUpRight(board,i,j);
			int opUpLeft =  numUpLeft(board,i,j);

			if (opUp+opDown>=2 || opLeft+opRight>=2 || opUpLeft+opDownRight>=2 || opUpRight+opDownLeft>=2){//checks if there are 3 or more opponent pieces in a row
                board[i][j] = whomoves;//makes the player move to the board
				pieces++;//increases the number of pieces
				noPiece = false;
				newkey = oldkey ^ R[i][j][0];//strips the empty value from the old key and assigns the new value to new key
				newkey ^= R[i][j][hash];//adds the piece value to the key array and then XOR's it into the new key
				if(transposition[newkey]){//checks if there is a value in the transposition table for the board state
					nn->child_[i][j]->score_ = transposition[newkey]; //assigns the score from the transposition
				}
				else{ //else creates a new child node
				nn->child_[i][j] =CreateTree(board,lvl+1,i,j,oprow,opcol,newkey,pieces);
				}
			}
			else{
			board[i][j] = whomoves;//places the players piece onto the board if the opponent does not have 3 or more in a row
			int pUp =  numUp(board,i,j);//checks the areas around the placed piece
			int pDown =  numDown(board,i,j);
			int pLeft =  numLeft(board,i,j);
			int pRight =  numRight(board,i,j);
			int pDownRight =  numDownRight(board,i,j);
			int pDownLeft  =  numDownLeft(board,i,j);
			int pUpRight   =  numUpRight(board,i,j);
			int pUpLeft =  numUpLeft(board,i,j);

			if((pUp+pDown>0 || pLeft+pRight>0 || pUpLeft+pDownRight>0 || pUpRight+pDownLeft>0) && pieces <20){//checks if there is a piece next to the one place and if the piece count is less than 20
				pieces++;//adds a piece
				noPiece = false;
				newkey = oldkey ^ R[i][j][0];//strips the empty value from the old key and assigns the new value to new key
				newkey ^= R[i][j][hash];//adds the piece value to the key array and then XOR's it into the new key
				if(transposition[newkey]){//checks if there is a value in the transposition table for the board state
					nn->child_[i][j]->score_ = transposition[newkey];//assigns the score from the transposition
				}
				else{//else creates a new child node
				nn->child_[i][j] =CreateTree(board,lvl+1,i,j,oprow,opcol,newkey,pieces);
				}

			}
			//checks if there are 2 pieces next to the piece and if the pieces on the board are greater than 20 and less than 40
			else if((pUp+pDown>1 || pLeft+pRight>1 || pUpLeft+pDownRight>1 || pUpRight+pDownLeft>1) && pieces >=20 && pieces <40){
				pieces++;//adds a piece
				noPiece = false;
				newkey = oldkey ^ R[i][j][0];//strips the empty value from the old key and assigns the new value to new key
				newkey ^= R[i][j][hash];//adds the piece value to the key array and then XOR's it into the new key
				if(transposition[newkey]){//checks if there is a value in the transposition table for the board state
					nn->child_[i][j]->score_ = transposition[newkey];//assigns the score from the transposition
				}
				else{//else creates a new child node
				nn->child_[i][j] =CreateTree(board,lvl+1,i,j,oprow,opcol,newkey,pieces);
				}
			}
			//checks if there are 3 pieces next to the piece and if the pieces on the board are greater than 40
			else if((pUp+pDown>2 || pLeft+pRight>2 || pUpLeft+pDownRight>2 || pUpRight+pDownLeft>2) && pieces >=40){
				pieces++;//adds a piece
				noPiece = false;
				newkey = oldkey ^ R[i][j][0];//strips the empty value from the old key and assigns the new value to new key
				newkey ^= R[i+1][j][hash];//adds the piece value to the key array and then XOR's it into the new key
				if(transposition[newkey]){//checks if there is a value in the transposition table for the board state
					nn->child_[i][j]->score_ = transposition[newkey];//assigns the score from the transposition
				}
				else{//else creates a new child node
				nn->child_[i][j] =CreateTree(board,lvl+1,i,j,oprow,opcol,newkey,pieces);
				}
			}
			
			if((pUp+pDown>0 || pLeft+pRight>0 || pUpLeft+pDownRight>0 || pUpRight+pDownLeft>0) && noPiece == true){//checks if there is a piece next to the one place and if the piece count is less than 20
				pieces++;//adds a piece
				noPiece = false;
				newkey = oldkey ^ R[i][j][0];//strips the empty value from the old key and assigns the new value to new key
				newkey ^= R[i][j][hash];//adds the piece value to the key array and then XOR's it into the new key
				if(transposition[newkey]){//checks if there is a value in the transposition table for the board state
					nn->child_[i][j]->score_ = transposition[newkey];//assigns the score from the transposition
				}
				else{//else creates a new child node
				nn->child_[i][j] =CreateTree(board,lvl+1,i,j,oprow,opcol,newkey,pieces);
				}

			}
			}
			board[i][j] = 0;//removes the piece from the board
			}


	}
	}
}
/*board is the board right now.  lvl is the lvl of the node we are creating,
  row and col is the position of the most recently added piece. row and col are
  not relevant when lvl is 0 and are invalid*/
//prow and pcol are the players last move, oldkey is the last generated hash key for the transposition table, and pieces is the number of pieces
dmmcgrath::Node* dmmcgrath::GameTree::CreateTree(int board[][NUMSQUARES],int lvl,int row,int col,int prow,int pcol, int oldkey,int pieces){
  Node* nn=new Node();
  int hash;
  if(player_ == -1){
	  hash = 2;
  }
  else{
	  hash = 1;
  }
  int whowon;
  int whomoves=(lvl%2)?-player_:player_;
  if(lvl > 0)
    whowon=fastCheckWin(board,row,col);
  else
    whowon=checkWin(board);

  //Base case 1: someone has won the game, game is over score is 100
  if(whowon) {
    nn->score_=whowon * player_* MAXSCORE; //if player_ is 1 and they won the whowon would be +ve so
                                           //we have score being +1, otherwise if player_ is -1 and 
                                           //player 2 won, then whowon would be -1 so score will still
                                           //be +1.
  }
  else if(boardFull(board)){     //tie game
    nn->score_=0;
  }
  else if(lvl==height_-1){  //game is tied.
	  //passes in row as the last move of the opponent and prow and pcol as the last move of the player
    nn->score_=Eval(board,player_,row,col,prow,pcol);    //always evaluate for player at root!
  }
  else{
       
		if(lvl == 0)
		generateMove(board,oprow,opcol,prow,pcol,lvl,nn,whomoves,oldkey,pieces);//if there are no childen use the values provided from the player class
		else
			generateMove(board,row,col,prow,pcol,lvl,nn,whomoves,oldkey,pieces);//pass in the row and col as the opponents last moves,prow and pcol as the players last moves
                                                                                //lvl is the lvl of the current node,nn is the current node, whomoves is whomoves
		                                                                        //oldkey is the previous hash key, and pieces is the number of pieces on the board
        
    bool setgood=false;

    //NOTE:  If you implement alpha beta pruning, you will have to track max/min in the
    //tree creation loop.
    if(lvl%2){
      for(int i=0;i<NUMSQUARES;i++){
        for(int j=0;j<NUMSQUARES;j++){
          if(nn->child_[i][j]){
            if(!setgood || nn->child_[i][j]->score_ < nn->score_){
				if(nn->child_[i][j]->score_ >= 2000000){//checks if score is equal to a sudo win and if it is subtracts lvl from its score so that a score higher up the tree is chosen first
					nn->child_[i][j]->score_ -= lvl;
				}
             nn->score_=nn->child_[i][j]->score_;
             setgood=true;
            }
			if(lvl >=2 && transposition.find(nn->child_[i][j]->key) == transposition.end()){//chcecks if the tree is at least 2 lvls deep and if a value already exists at that key position
				transposition[nn->child_[i][j]->key] = nn->child_[i][j]->score_;//adds a value to the transposition table at the index of the childs key
			}
          }
        }//for
      }//for
    }
    else{
      for(int i=0;i<NUMSQUARES;i++){
        for(int j=0;j<NUMSQUARES;j++){
          if(nn->child_[i][j]){
            if(!setgood || nn->child_[i][j]->score_ > nn->score_){
				if(nn->child_[i][j]->score_ >= 2000000){//checks if score is equal to a sudo win and if it is subtracts lvl from its score so that a score higher up the tree is chosen first
					nn->child_[i][j]->score_ -= lvl;
				}
              nn->score_=nn->child_[i][j]->score_;
              setgood=true;
              if(lvl==0){
				  int test = nn->score_;
                bestrow_=i;
                bestcol_=j;
              }
            }
			if(lvl >=2 && transposition.find(nn->child_[i][j]->key) == transposition.end()){//chcecks if the tree is at least 2 lvls deep and if a value already exists at that key position
				transposition[nn->child_[i][j]->key] = nn->child_[i][j]->score_;//adds a value to the transposition table at the index of the childs key
			}
          }
        }//for
      }//for
    }
  }
  return nn;
}
/*PLEASE PLEASE PLEASE REMEMBER YOUR DESTRUCTORS!  with the size of the trees
  we are creating, a lack of one will be a major issue*/
dmmcgrath::GameTree::~GameTree(){
  DestroyTree(root_);
}

void dmmcgrath::GameTree::DestroyTree(Node* root){
  if (root){
    for(int i=0;i<NUMSQUARES;i++){
      for(int j=0;j<NUMSQUARES;j++){
        DestroyTree(root->child_[i][j]);
      }
    }
    delete root;
  }/*if*/
}
/*this function returns a board score for the given board for the player
  indicated.*/
// passes in the player to evaluate for, the opponents row and col and the players row and col
int dmmcgrath::Eval(int board[][NUMSQUARES],int player,int oprow,int opcol,int prow, int pcol){
  int p1score= 0;  //range 1 to MAXSCORE -1
  int p2score=0;  //range 1 to MAXSCORE -1
  int totalscore = 0;

  	int opDown=0, opUp=0,opLeft=0,opRight=0,opDownRight=0,opUpLeft=0,opUpRight=0,opDownLeft=0;//used for getting the number of pieces around the opponents piece
	int pDown=0, pUp=0,pLeft=0,pRight=0,pDownRight=0,pUpLeft=0,pUpRight=0,pDownLeft=0;//used for getting the number of pieces around the players piece
	int opPiece;
	if(player == 1)//assigns the value of the opposing players piece
		opPiece = -1;
	else
		opPiece = 1;
	p1score = 1000;
	p2score = 1000;
	opDown = numDown(board,oprow,opcol);//gets the number of opponent pieces up and down
	opUp = numUp(board,oprow,opcol);
	if(opPiece == 1){
		if(((opDown+opUp)+1) ==2)//assigns a score based on how many pieces are in a row
			p1score += 2000;
		else if(((opDown+opUp)+1) ==3)
			p1score += 5000;
		else if(((opDown+opUp)+1) ==4)
			p1score += 10000;

	}
	else{
		if(((opDown+opUp)+1) ==2)//assigns a score based on how many pieces are in a row
			p2score += 2000;
		else if(((opDown+opUp)+1) ==3)
			p2score += 5000;
		else if(((opDown+opUp)+1) ==4)
			p2score += 30000;
	}
	//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[(oprow+opDown)+1][opcol] == player && opPiece ==1 && (opDown+opUp) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[(oprow-opUp)-1][opcol] == player && opPiece ==1 && (opDown+opUp) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[(oprow+opDown)+1][opcol] == player && opPiece ==-1 && (opDown+opUp) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[(oprow-opUp)-1][opcol] == player && opPiece ==-1 && (opDown+opUp) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}
	opLeft = numLeft(board,oprow,opcol);//checks the number of pieces to the right and left
	opRight = numRight(board,oprow,opcol);
	if(opPiece == 1){
		if(((opLeft+opRight)+1) ==2)//assigns a score based on how many pieces are in a row
			p1score += 2000;
		else if(((opLeft+opRight)+1) ==3)
			p1score += 5000;
		else if(((opLeft+opRight)+1) ==4)
			p1score += 30000;

	}
	else{
		if(((opLeft+opRight)+1) ==2)//assigns a score based on how many pieces are in a row
			p2score += 2000;
		else if(((opLeft+opRight)+1) ==3)
			p2score += 5000;
		else if(((opLeft+opRight)+1) ==4)
			p2score += 30000;
	}
	//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[oprow][(opcol-opLeft)-1] == player && opPiece ==1 && (opLeft+opRight) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[oprow][(opcol+opRight)+1] == player &&opPiece ==1 && (opLeft+opRight) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[oprow][(opcol-opLeft)-1] == player && opPiece ==-1 && (opLeft+opRight) >=2){
		p2score = p2score - 4000;
		p1score += 50000;
	}//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[oprow][(opcol+opRight)+1] == player && opPiece ==-1 && (opLeft+opRight) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}
	opDownRight = numDownRight(board,oprow,opcol);//checks the number of pieces diagonally to the upper left
	opUpLeft = numUpLeft(board,oprow,opcol);

	if(opPiece == 1){
		if(((opDownRight+opUpLeft)+1) ==2)//assigns a score based on how many pieces are in a row
			p1score += 2000;
		else if(((opDownRight+opUpLeft)+1) ==3)
			p1score += 5000;
		else if(((opDownRight+opUpLeft)+1) ==4)
			p1score += 30000;

	}
	else{
		if(((opDownRight+opUpLeft)+1) ==2)//assigns a score based on how many pieces are in a row
			p2score += 2000;
		else if(((opDownRight+opUpLeft)+1) ==3)
			p2score += 5000;
		else if(((opDownRight+opUpLeft)+1) ==4)
			p2score += 30000;
	}
	//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[(oprow+opDownRight)+1][(opcol+opDownRight)+1] == player && opPiece ==1 && (opDownRight+opUpLeft) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[(oprow-opUpLeft)-1][(opcol-opUpLeft)-1] == player &&opPiece ==1 && (opDownRight+opUpLeft) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[(oprow+opDownRight)+1][(opcol+opDownRight)+1] == player && opPiece ==-1 && (opDownRight+opUpLeft) >=2){
	p2score = p2score - 4000;
	    p1score += 50000;
	}//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[(oprow-opUpLeft)-1][(opcol-opUpLeft)-1] == player && opPiece ==-1 && (opDownRight+opUpLeft) >=2){
		p2score = p2score - 4000;
	    p1score += 500000;
	}
	opUpRight = numUpRight(board,oprow,opcol);//checks the number of pieces diagonally to the upper right and lower left
	opDownLeft = numDownLeft(board,oprow,opcol);

	if(opPiece == 1){
		if(((opDownLeft+opUpRight)+1) ==2)//assigns a score based on how many pieces are in a row
			p1score += 2000;
		else if(((opDownLeft+opUpRight)+1) ==3)
			p1score += 5000;
		else if(((opDownLeft+opUpRight)+1) ==4)
			p1score += 30000;

	}
	else{
		if(((opDownLeft+opUpRight)+1) ==2)//assigns a score based on how many pieces are in a row
			p2score += 2000;
		else if(((opDownLeft+opUpRight)+1) ==3)
			p2score += 5000;
		else if(((opDownLeft+opUpRight)+1) ==4)
			p2score += 30000;
	}
	//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[(oprow-opUpRight)-1][(opcol+opUpRight)+1] == player && opPiece ==1 && (opDownLeft+opUpRight) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[(oprow+opDownLeft)+1][(opcol-opDownLeft)-1] == player &&opPiece ==1 && (opDownLeft+opUpRight) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[(oprow-opUpRight)-1][(opcol+opUpRight)+1] == player && opPiece ==-1 && (opDownLeft+opUpRight) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}//checks if the opponent has 3 or more in a row and assigns a value to the player for blocking and subtracs from the opponent for being blocked
	if(board[(oprow+opDownLeft)+1][(opcol-opDownLeft)-1] == player && opPiece ==-1 && (opDownLeft+opUpRight) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}
	//end of checking pieces around opponent pieces

	pDown = numDown(board,prow,pcol);//checks the number of player pieces up and down
	pUp = numUp(board,prow,pcol);

	if(player == 1){
		if(((pDown+pUp)+1) ==2)//assigns a score based on how many pieces are in a row
			p1score += 2000;
		else if(((pDown+pUp)+1) ==3)
			p1score += 5000;
		else if(((pDown+pUp)+1) ==4)
			p1score += 30000;

	}
	else{
		if(((pDown+pUp)+1) ==2)//assigns a score based on how many pieces are in a row
			p2score += 2000;
		else if(((pDown+pUp)+1) ==3)
			p2score += 5000;
		else if(((pDown+pUp)+1) ==4)
			p2score += 30000;
	}
	//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[(prow+pDown)+1][pcol] == opPiece && player ==1&& (pUp+pDown) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[(prow-pUp)-1][pcol] == opPiece && player ==1&& (pUp+pDown) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[(prow+pDown)+1][pcol] == opPiece && player ==-1&& (pUp+pDown) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[(prow-pUp)-1][pcol] == opPiece && player ==-1&& (pUp+pDown) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}
	pLeft = numLeft(board,prow,pcol);//checks the player pieces to the left and right
	pRight = numRight(board,prow,pcol);

	if(player == 1){
		if(((pLeft+pRight)+1) ==2)//assigns a score based on how many pieces are in a row
			p1score += 2000;
		else if(((pLeft+pRight)+1) ==3)
			p1score += 5000;
		else if(((pLeft+pRight)+1) ==4)
			p1score += 30000;

	}
	else{
		if(((pLeft+pRight)+1) ==2)//assigns a score based on how many pieces are in a row
			p2score += 2000;
		else if(((pLeft+pRight)+1) ==3)
			p2score += 5000;
		else if(((pLeft+pRight)+1) ==4)
			p2score += 30000;
	}
	//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[prow][(pcol+pRight)+1] == opPiece && player ==1&& (pRight+pLeft) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[prow][(pcol-pLeft)-1] == opPiece && player ==1&& (pRight+pLeft) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[prow][(pcol+pRight)+1] == opPiece && player ==-1&& (pRight+pLeft) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[prow][(pcol-pLeft)-1] == opPiece && player ==-1 && (pRight+pLeft) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}
	pDownRight = numDownRight(board,prow,pcol);//checks the player pieces diagonally to the lower right and upper left
	pUpLeft = numUpLeft(board,prow,pcol);

	if(player == 1){
		if(((pDownRight+pUpLeft)+1) ==2)//assigns a score based on how many pieces are in a row
			p1score += 2000;
		else if(((pDownRight+pUpLeft)+1) ==3)
			p1score += 5000;
		else if(((pDownRight+pUpLeft)+1) ==4)
			p1score += 30000;

	}
	else{
		if(((pDownRight+pUpLeft)+1) ==2)//assigns a score based on how many pieces are in a row
			p2score += 2000;
		else if(((pDownRight+pUpLeft)+1) ==3)
			p2score += 5000;
		else if(((pDownRight+pUpLeft)+1) ==4)
			p2score += 30000;
	}
	//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[(prow+pDownRight)+1][(pcol+pDownRight)+1] == opPiece && player ==1 && (pUpLeft+pDownRight) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[(prow-pUpLeft)-1][(pcol-pUpLeft)-1] == opPiece && player ==1 && (pUpLeft+pDownRight) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[(prow+pDownRight)+1][(pcol+pDownRight)+1] == opPiece && player ==-1 && (pUpLeft+pDownRight) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[(prow-pUpLeft)-1][(pcol-pUpLeft)-1] == opPiece && player ==-1 && (pUpLeft+pDownRight) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}
	pUpRight = numUpRight(board,prow,pcol);//checks the player pieces diagonally to the upper right and lower left
	pDownLeft = numDownLeft(board,prow,pcol);

	if(player == 1){
		if(((pDownLeft+pUpRight)+1) ==2)//assigns a score based on how many pieces are in a row
			p1score += 2000;
		else if(((pDownLeft+pUpRight)+1) ==3)
			p1score += 5000;
		else if(((pDownLeft+pUpRight)+1) ==4)
			p1score += 30000;

	}
	else{
		if(((pDownLeft+pUpRight)+1) ==2)//assigns a score based on how many pieces are in a row
			p2score += 2000;
		else if(((pDownLeft+pUpRight)+1) ==3)
			p2score += 5000;
		else if(((pDownLeft+pUpRight)+1) ==4)
			p2score += 30000;
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[(prow-pUpRight)-1][(pcol+pUpRight)+1] == opPiece && player ==1 && (pUpRight+pDownLeft) >=2){
		p1score = p1score - 4000;
		p2score += 50000;  
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[(prow+pDownLeft)+1][(pcol-pDownLeft)-1] == opPiece && player ==1 && (pUpRight+pDownLeft) >=2){
		p1score = p1score - 4000;
		p2score += 50000;
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[(prow-pUpRight)-1][(pcol+pUpRight)+1] == opPiece && player ==-1 && (pUpRight+pDownLeft) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}//checks if the player has 3 or more in a row and assigns a value to the opponent for blocking and subtracs from the player for being blocked
	if(board[(prow+pDownLeft)+1][(pcol-pDownLeft)-1] == opPiece && player ==-1 && (pUpRight+pDownLeft) >=2){
		p2score = p2score - 4000;
	    p1score += 50000;
	}
	
	if(fastCheckWin(board,prow,pcol) == 1)//assigns a sudo win to player 1
		p1score = 2000000;
	if(fastCheckWin(board,prow,pcol) ==-1)//assigns a sudo win to player 2
		p2score = 2000000;
	if((fastCheckWin(board,oprow,opcol) == 1) && opPiece == 1)//assigns a sudo win to player 1
		p1score = 2000000;
	if((fastCheckWin(board,oprow,opcol) ==-1) && opPiece == -1)//assigns a sudo win to player 2
		p2score = 2000000;

  if (player==1){
    totalscore=p1score-p2score;
  }/*if*/
  else{
    totalscore=p2score-p1score;
  }
  return totalscore;
}